/* +-------------------------------------------------------------------+ */
/* |  IMCC.csi.ull.es Curso 1998-99                                    | */
/* |  pnm2pat.c                                                        | */
/* |                                                                   | */
/* |  Transforma un conjunto de ficheros graficos pnm (pbm, pgm o ppm) | */
/* |  a un fichero de patrones snns.                                   | */
/* |  Usar conjuntamente con mkout, mkhead y/o mkpat.                  | */
/* |                                                                   | */
/* +-------------------------------------------------------------------+ */


/* +-------------------------------------------------------------------+ */
/* | Copyright 1992, 1993, David Koblas (koblas@netcom.com)	       | */
/* |								       | */
/* | Permission to use, copy, modify, and to distribute this software  | */
/* | and its documentation for any purpose is hereby granted without   | */
/* | fee, provided that the above copyright notice appear in all       | */
/* | copies and that both that copyright notice and this permission    | */
/* | notice appear in supporting documentation.	 There is no	       | */
/* | representations about the suitability of this software for	       | */
/* | any purpose.  this software is provided "as is" without express   | */
/* | or implied warranty.					       | */
/* |								       | */
/* +-------------------------------------------------------------------+ */


#include <stdio.h>
#include <time.h>
#include "libpnmrw.h"

#define MAX_SIZE 255
#define NUM_LINE 10
#define TRUE 1
#define FALSE 0

char *program;

void error(char *msg)
{
    fprintf(stderr,"%s: %s\n", program, msg);
}

unsigned int num_pat;    /* Nmero de patrones */
int cab = 1;             /* Imprime cabecera? */

void printcab(FILE *fstream, unsigned int size)
{
    time_t t;

    if (cab) {
	time(&t);

	fprintf(fstream, "SNNS pattern definition file V1.4\n");
	fprintf(fstream, "generated at %s\n\n", ctime( &t));
	fprintf(fstream, "No. of patterns : %d\n", num_pat);
	fprintf(fstream, "No. of input units : %d\n", size);
	fprintf(fstream, "No. of output units : 0\n\n");
    }
}


int ReadPNM(FILE *fd, char *name, FILE *fstreamout, unsigned int *wwin, unsigned int *hhin)
{
    int x, y;
    xel **els = NULL;
    xel *curel;
    xelval maxval;
    int pnm_type;
    int format;
    int wth, hth;
    unsigned char *dp;
    char tipcar;

    pnm_init2(name);
    if ((els = pnm_readpnm(fd, &wth, &hth, &maxval, &format)) == NULL) {
	error("formato de fichero inadecuado");
	return 1;
    }
    pnm_type = PNM_FORMAT_TYPE(format);

    if ((*wwin == 0) && (*hhin == 0)) {
	*wwin = wth;
	*hhin = hth;
	printcab(fstreamout, (wth * hth) * ((pnm_type == PPM_TYPE)?3:1));
    } else
	if ((*wwin != wth) || (*hhin != hth)) {
	    error("Alto y ancho no coinciden entre los distintos ficheros");
	    return 1;
	}

    switch (pnm_type) {
    case PPM_TYPE: 
	tipcar='p';
	break;
    case PGM_TYPE:
	tipcar='g';
	break;
    case PBM_TYPE:
	tipcar='b';
	break;
    }

    fprintf(fstreamout, "\n#Input pattern %s (p%cm %dx%d):\n", name, tipcar,wth, hth);

    for (y = 0; y < hth; y++) {
	for (curel = els[y], x = 0; x < wth; curel++) {
	    x++;
	    if (pnm_type == PPM_TYPE) {
		fprintf(fstreamout, "%.5f%c", (float) PPM_GETR(*curel) / maxval , ((x % NUM_LINE)?' ':'\n'));
		fprintf(fstreamout, "%.5f%c", (float) PPM_GETG(*curel) / maxval , ((x % NUM_LINE)?' ':'\n'));
		fprintf(fstreamout, "%.5f%c", (float) PPM_GETB(*curel) / maxval , ((x % NUM_LINE)?' ':'\n'));
	    } else if (pnm_type == PGM_TYPE) {
		fprintf(fstreamout, "%.5f%c", (float) PNM_GET1(*curel) / maxval , ((x % NUM_LINE)?' ':'\n'));
	    } else {
		fprintf(fstreamout, "%d%c",  (PNM_GET1(*curel) != PBM_WHITE) ? 1 : 0, ((x % NUM_LINE)?' ':'\n'));
	    }
	}
    }

    pnm_freearray(els, hth);

    return 0;
}


void help()
{

  fprintf(stderr, "Uso: %s [-c] [-o fdest] file1 file2 ... filen\n", program);
  fprintf(stderr, "Opciones:\n");
  fprintf(stderr, "\t-c: No incluye cabecera de patrones\n");
  fprintf(stderr, "\t-o fdest: Salida hacia el fichero fdest\n");
  exit(-1);
}



int main(int argc, char **argv)
{
  extern char *optarg;
  extern int optind;
  char c;


  char filedest[MAX_SIZE];
  FILE *fi;
  FILE *fh = stdout;

  unsigned int ww = 0;	/* width */
  unsigned int hh = 0;	/* height */

  program = argv[0];

  while ((c = getopt(argc, argv, "co:h")) != -1)
    switch (c) {
    case 'c':
      cab = !cab;
      break;
    case 'o':
      sscanf(optarg, "%s", &filedest);
      if ((fh = fopen(filedest, "w"))==NULL) {
	perror(program);
	return (1);
      }
      break;
    case 'h':
      help();
      return(0);
      break;
    default:
      error("parametros inadecuados");
      return(1);
  }

  if (optind >= argc) {
    num_pat = 1;
    fi = stdin;

    if (ReadPNM(fi, "stdin", fh, &ww, &hh) != 0) {
      return(1);
    }
  } else {
    num_pat = argc - optind;
    while (optind < argc) {
	if ((fi = fopen(argv[optind], "r")) == NULL) {
	    perror(program);
	    return(1);
	}

	if (ReadPNM(fi, argv[optind], fh, &ww, &hh) != 0) {
	    return(1);
	}
	fclose(fi);
	optind++;
    }
  }

  return (0);
}
  
